/*********************************************************************************
 * 	PURPOSE:	Rotate Stepper Motor and Turn on LED Matrix with Remote Control 
 * 	Created by      Rebecca Ellis
 * 	DATE:		11/2014
 **********************************************************************************/

#include <Adafruit_GFX.h>
#include <Adafruit_NeoMatrix.h>
#include <Adafruit_NeoPixel.h>
#include <StepperAK.h>
#include <IRremote.h>


#define PIN 6 //LED Matrix input on Pin 6
#define gearratio 64 //1:64 gear ratio for stepper motor
#define BOARDCOUNT 1   // number of LED matrices
Adafruit_NeoPixel strip = Adafruit_NeoPixel(64 * BOARDCOUNT, PIN, NEO_GRB + NEO_KHZ800); //Sets up the number of LED nodes
double brightness = .1; // reduce brightness for the LED Matrix

const int stepsPerRevolution = 2048;  //2048 steps turn the motor one revolution
Stepper myStepper(stepsPerRevolution, A0, A1, A2, A3); //  4-wire stepper on pins A0-A3

int RECV_PIN = 9; //IR Receiver connected to Pin 9
IRrecv irrecv(RECV_PIN); //Create a IR receiver object
decode_results results; //Create a decode_results object

Adafruit_NeoMatrix matrix = Adafruit_NeoMatrix(8, 8, PIN,  //Initiates an 8x8 LED Matrix
NEO_MATRIX_TOP     + NEO_MATRIX_RIGHT +
NEO_MATRIX_COLUMNS + NEO_MATRIX_PROGRESSIVE,
NEO_GRB            + NEO_KHZ800);

const uint16_t colors[] = {
  matrix.Color(255, 0, 0), matrix.Color(0, 255, 0), matrix.Color(0, 0, 255) };  //Defines LED Matrix Colors

void setup() {

  myStepper.setSpeed(0.15*gearratio);  //the motor appears to be geared down 1/64, meaning that the speed needs to be set 64x.
  irrecv.enableIRIn(); //Enables the IR device
  matrix.begin(); //Starts the LED Matrix with a blank display
  matrix.setTextWrap(false);
  matrix.setBrightness(40);
  matrix.setTextColor(colors[0]);
}

int x    = matrix.width(); //Establishes X as the matrix width
int pass = 0; //Establishes pass variable 
int flag=0; //variable to establish when the device is powered on via remote
void loop() {
  
    if (irrecv.decode(&results)) {//has a transmission been received?
      if (results.value == 3584887633){  //If the power button is pressed the flag variable changes and the main program executes
        flag=1; 
      }
    }
        //irrecv.resume(); // Receive the next value from the IR Sensor

if (flag==1) //If flag is equal to 1, then the power button was pressed
{
  
  
  if (irrecv.decode(&results)) {//has a transmission been received?
    Serial.println(results.value);//If yes: interpret the received commands...
    if (results.value == 2354352215){ //If the "-" Button is pressed do the following:
      matrix.fillScreen(0);  //Clears the LED matrix
      matrix.setCursor(2, 0); //Sets the cursor position
      matrix.setTextColor(colors[2]); //Sets the text color
      matrix.print(F("?")); //Prints the "?" character to the screen
      matrix.show(); //Enables the matrix
      delay(100); //Delays the matrix
      myStepper.step(stepsPerRevolution); //Rotates the stepper motor

    }
    if (results.value == 3730122487){  //If the "+" button on the remote is pressed the following executes.  Similar to above
      matrix.fillScreen(0);
      matrix.setCursor(2, 0);
      matrix.setTextColor(colors[1]);
      matrix.print(F("?"));
      matrix.show();
      delay(100);
      myStepper.step(-stepsPerRevolution);  //Rotates the stepper motor backwards.
    }
    if (results.value == 3584887633){  //If the power button is pressed a Rainbow lights up
      colorWipe(strip.Color(brightness * 255, 0, 0), 5); // Red
      colorWipe(strip.Color(0, brightness * 255, 0), 5); // Green
      colorWipe(strip.Color(0, 0, brightness * 255), 5); // Blue
      rainbow(5);
      rainbowCycle(5);
    }
    irrecv.resume(); // Receive the next value from the IR Sensor
  } 
  matrix.fillScreen(0);  //If no input on the remote, clear the matrix screen
  matrix.setCursor(x, 0); //Sets the cursor position to variable 'x'
  matrix.print(F("MAKE")); //Prints the word "Make" to the matrix
  if(--x < -36) { //Cycles the cursor position to make text appear to scroll
    x = matrix.width();
    if(++pass >= 3) pass = 0; //Cycles through three colors for the text display
    matrix.setTextColor(colors[pass]);
  }
  matrix.show(); //Prints text to matrix
  delay(100); //Establishes a delay

}
}

// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
    strip.setPixelColor(i, c);
    strip.show();
    delay(wait);
  }
}

void rainbow(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256; j++) {
    for(i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i,Wheel((i+j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}
// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  if(WheelPos < 85) {
    return strip.Color(brightness * WheelPos * 3, brightness * (255 - WheelPos * 3), 0);
  } 
  else if(WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(brightness * (255 - WheelPos * 3), 0, brightness * WheelPos * 3);
  } 
  else {
    WheelPos -= 170;
    return strip.Color(0, brightness * WheelPos * 3, brightness * (255 - WheelPos * 3));
  }
}

